/* 
   v4l1_core.c (C) Justin Schoeman 2001 (justin@suntiger.ee.up.ac.za)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <linux/videodev.h>
#include <pthread.h>

#include "nvrec.h"

#if (defined __LINUX_VIDEODEV2_H) || (defined V4L2_MAJOR_VERSION)

void v4lgen_core_usage()
{
    v4l1_core.cap_core_usage();
    v4l2_core.cap_core_usage();
}

struct cap_core_t *v4lgen_core_init(char **args, unsigned int **pfmt)
{
    struct cap_core_t *cct = NULL;

    struct v4l2_capability v4l2cap;
    struct v4l2_tuner v4l2tuner;

    struct video_capability v4l1cap;
    char *fdev = NULL;
    char *inp = NULL;
    int fd = -1;
    NV_ARGS;

    NV_SCAN_ARGS {
	if (strcmp(NV_THIS_ARG, "-v") == 0) {
	    if (!NV_NEXT_ARG) {
		EPRINT("Missing argument to -v.\n");
		goto usage;
	    }
	    fdev = NV_THIS_ARG;
	} else if (strcmp(NV_THIS_ARG, "-input") == 0) {
	    if (!NV_NEXT_ARG) {
		EPRINT("Missing argument to -v.\n");
		goto usage;
	    }
	    inp = NV_THIS_ARG;
	}

    }
    IPRINT("Device %s\n", fdev);
    /* Radio devices */
    if (strcmp(inp, "radio") == 0) {
	fd = open(fdev, O_RDONLY);
	if (fd < 0) {
	    PERROR("Cannot open radio device");
	    goto error;
	}
#ifndef V4L2_MAJOR_VERSION	
	if ((ioctl(fd, VIDIOC_QUERYCAP, &v4l2cap) == 0) &&
	    (ioctl(fd, VIDIOC_G_TUNER, &v4l2tuner) == 0)) {
	    if ((v4l2cap.capabilities & V4L2_CAP_TUNER) &&
		(v4l2tuner.type & V4L2_TUNER_RADIO)) {
		close(fd);
		fd = -1;
		cct = v4l2_radio_core.cap_core_init(args, pfmt);
		if (cct) {
		    IPRINT("Got capable v4l2 device: '%s' (driver:'%s').\n", v4l2cap.driver, v4l2cap.card);
		    return cct;
		}
	    }
	}
#else

	if (ioctl(fd, VIDIOC_QUERYCAP, &v4l2cap) == 0) {
	    if ((v4l2cap.type == V4L2_TYPE_RADIO) &&
		(v4l2cap.flags & V4L2_FLAG_TUNER)) {
		close(fd);
		fd = -1;
		cct = v4l2_radio_core.cap_core_init(args, pfmt);
		if (cct) {
		    IPRINT("Got capable v4l2 device: %s.\n", v4l2cap.name);
		    return cct;
		}
	    }
	}

#endif
	/* At this point, we haven't a v4l2 radio tuner device */
	if (fd < 0)
	    fd = open(fdev, O_RDONLY);

	if (fd < 0) {
	    PERROR("Cannot open capture device");
	    goto error;
	}
	if (ioctl(fd, VIDIOCGCAP, &v4l1cap) == 0) {
	    if (v4l1cap.type & VID_TYPE_TUNER) {
		close(fd);
		fd = -1;
		cct = v4l1_radio_core.cap_core_init(args, pfmt);
		if (cct) {
		    IPRINT("Got capable v4l1 device: %s.\n", v4l1cap.name);
		    return cct;
		}
	    }
	}

	EPRINT("Could not get any capable capture devices.\n");

    } else {
	/* Video devices */

	fd = open(fdev, O_RDWR);
	if (fd < 0) {
	    PERROR("Cannot open capture device");
	    goto error;
	}
#ifndef V4L2_MAJOR_VERSION
	if (ioctl(fd, VIDIOC_QUERYCAP, &v4l2cap) == 0) {
	    if ((v4l2cap.capabilities & V4L2_CAP_VIDEO_CAPTURE) &&
		(v4l2cap.capabilities & V4L2_CAP_STREAMING)) {
		close(fd);
		fd = -1;
		cct = v4l2_core.cap_core_init(args, pfmt);

		if (cct) {
		    IPRINT
			("Got capable v4l2 device: '%s' (driver:'%s').\n",
			 v4l2cap.driver, v4l2cap.card);
		    return cct;
		}
	    }
	}
#else
	if (ioctl(fd, VIDIOC_QUERYCAP, &v4l2cap) == 0) {
	    if ((v4l2cap.type == V4L2_TYPE_CAPTURE) &&
		(v4l2cap.flags & V4L2_FLAG_STREAMING)) {
		close(fd);
		fd = -1;
		cct = v4l2_core.cap_core_init(args, pfmt);
		if (cct) {
		    IPRINT("Got capable v4l2 device: %s.\n", v4l2cap.name);
		    return cct;
		}
	    }
	}
#endif

	if (fd < 0)
	    fd = open(fdev, O_RDWR);
	if (fd < 0) {
	    PERROR("Cannot open capture device");
	    goto error;
	}
	if (ioctl(fd, VIDIOCGCAP, &v4l1cap) == 0) {
	    if (v4l1cap.type & VID_TYPE_CAPTURE) {
		close(fd);
		fd = -1;
		cct = v4l1_core.cap_core_init(args, pfmt);
		if (cct) {
		    IPRINT("Got capable v4l1 device: %s.\n", v4l1cap.name);
		    return cct;
		}
	    }
	}

	EPRINT("Could not get any capable capture devices.\n");
      error:
	if (fd >= 0)
	    close(fd);
	return NULL;
      usage:
	EPRINT
	    ("Error parsing options.  Please run without any command line options for\n");
	EPRINT("detailed usage information.\n");
	return NULL;
    }
    /* If we reaches that point no capture can be used */
    return NULL;
}

#else

struct cap_core_t *v4lgen_core_init(char **args, unsigned int **pfmt)
{
    NV_ARGS;
    char *inp = NULL;

    NV_SCAN_ARGS {
	if (strcmp(NV_THIS_ARG, "-input") == 0) {
	    if (!NV_NEXT_ARG) {
		EPRINT("Missing argument to -input.\n");
		goto usage;
	    }
	    inp = NV_THIS_ARG;
	}

    }
    /* Radio devices */
    if (strcmp(inp, "radio") == 0) {
	DPRINT("Radio device\n");
	return v4l1_radio_core.cap_core_init(args, pfmt);

    } else {
	DPRINT("Video device\n");
	return v4l1_core.cap_core_init(args, pfmt);
    }

  usage:
    EPRINT
	("Error parsing options.  Please run without any command line options for\n");
    EPRINT("detailed usage information.\n");
    return NULL;
}


void v4lgen_core_usage()
{
    return v4l1_core.cap_core_usage();
}

#endif

struct cap_core_t v4lgen_core = {
  cap_core_init:v4lgen_core_init,
  cap_core_close:NULL,
  cap_core_next:NULL,
  cap_core_status:NULL,
  cap_core_ftime:NULL,
  cap_core_overrides:NULL,
  cap_core_usage:v4lgen_core_usage,
  priv:NULL
};
